#!/usr/bin/env python

"""
A generic HTTP server class and a generic mail server handler

Should handle most search engines that accept URL's via web page forms.
"""

import serverbase
import time,string
import urllib
import smtp # custom local library

# Load translation tables
try:
    execfile("src/translate.py")
except:
    print "Error loading translation tables in generic.py! Aborting. :-("
    raise SystemExit(1)


class GenericHTTP(serverbase.ServerObjectBase):

    def __init__(self, G, name, config):
        serverbase.ServerObjectBase.__init__(self, G, name)

        SEConfigFailure = "There was an error configuring search engine " + name

        # required configuration info
        try:
            self.name = config.name
            self.machine = config.machine
            self.uri = config.uri
            self.FormQuery = config.FormQuery
            self.Success = config.Success
            self.Fail = config.Fail
            self.remote_port = config.port
            self.Params = config.Params
        except:
            raise SEConfigFailure

        try:
            self.PrintAll = config.PrintAll
        except:
            self.PrintAll = None

        try:
            self.EmailQuery = config.EmailQuery
        except:
            self.EmailQuery = None
            
        self.G.pprint(_("d_server_configured") % (self.name, self.machine, self.remote_port, self.uri), 2)

    def Go(self, URLList):
        for url in URLList:
            try:
                self.submitURL(url)
            except:
                self.G.pprint(_("unhandled_exception") % (self.name, url))

            
    def submitURL(self, URL):

        # add the URL to the query
        self.Params[self.FormQuery] = URL

        # optionally add an email address (the submiter's) to the query
        if self.EmailQuery:
            self.Params[self.EmailQuery] = self.G.email

        # start the timer
        starttime = time.time()
        self.results = None

        # DO IT!
        self.GetURL()

        # end the timer
        endtime = time.time()
        self.time = endtime - starttime
        self.timeaccum = self.timeaccum + self.time
        self.hitcount = self.hitcount + 1 # accumulate hits

        # check the results
        if self.results:
            if self.PrintAll:
                self.G.pprint(self.results)
            
            for message in self.Success:
                if string.find(self.results, message) > 0:
                    self.G.pprint(_("submit_success") % (self.name, URL))
                    self.successcount = self.successcount + 1
                    return
            
            # definetely failure at this point
            # but go ahead and look for explicit failure msgs
            
            self.errorcount = self.errorcount + 1
            
            for message in self.Fail:
                if string.find(self.results, message) > 0:
                    self.G.pprint(_("submit_fail") % (self.name, URL, message))
                    return

            self.G.pprint(_("generic_fail") % (self.name, URL))

            return
        
        else:
            self.G.pprint(_("no_results_fail" % (self.name, URL), 0))
            self.errorcount = self.errorcount + 1

    def GetURL(self):
        done = None
        self.counter = 0

        while self.counter <= self.retries and not done:
            try:
                ProxyDict = None
                if self.G.proxy:
                    if self.G.proxyport:
                        port = self.G.proxyport
                    else:
                        port = 80

                    ProxyDict = { "http" : "%s:%s" % (self.G.proxy, port)}
                    
                    self.G.pprint("Activating proxy host %s" % (ProxyDict["http"]), 1)
                else:
                    self.G.pprint("Not using proxy.", 2)

                request = "http://%s:%s%s?%s" % (self.machine, self.remote_port, self.uri, urllib.urlencode(self.Params))

                self.G.pprint(_("d_request_submit") % (request, self.name), 1)
                
                self.results = urllib.FancyURLopener(ProxyDict).open(request).read()

                self.G.pprint(_("d_request_complete") % (self.name), 1)
                    
                done = 1
            except IOError:
                self.counter = self.counter + 1
                self.G.pprint(_("d_retrying") % (self.name, self.counter), 2)
                time.sleep(1) # sleep for 1 sec between retries (this helps)

        if not done:
            self.G.pprint(_("network_error") % self.name)

            self.results = None
            self.errorcount = self.errorcount + 1                
        

class GenericMail(serverbase.ServerObjectBase):
    def __init__(self, G, name, config):
        serverbase.ServerObjectBase.__init__(self, G, name)
        try:
            self.server = G.smtp_server
            self.from_addr = G.email
            self.subject = config.subject
            self.to_addr = config.mailto
        except:
            import traceback
            traceback.print_exc()
            print "Can't configure mail server"
            raise SystemExit

    def Go(self, URLList):

        self.successcount = len(URLList)
        self.errorcount = 0

        Body = string.join(URLList)

        mail_handler = smtp.mail_handler(self.server, self.from_addr)

        results = mail_handler.send_email(self.to_addr, self.subject, Body)
    
        # log success
        self.G.pprint(_("mail_success") % (self.name, self.successcount, self.to_addr), 0)
        
