#!/usr/bin/env python

"""
This is a doc string for the Pagecast GTK module.
"""

from gtk import *
import GtkExtra
import submitengine
import string
from threading import *

class Pagecast(Thread):
    def __init__(self, SubmitList, ReplyTo = None, Hooks = None, Debug = None):
        Thread.__init__(self)
        self.SubmitList = SubmitList
        self.ReplyTo = ReplyTo
        self.Hooks = Hooks
        self.Debug = None
        self.setName("Pagecast GTK control")

    def run(self):
        se = submitengine.submitengine(self.SubmitList, self.ReplyTo, self.Hooks, self.Debug)
        
        # do that magic thang
        se.Go()
    
    def join(self):
        pass

class Application(GtkWindow):
    def __init__(self, name):
        GtkWindow.__init__(self, WINDOW_TOPLEVEL)
        self.connect("destroy", self.quit)
        self.connect("delete_event", self.quit)
        self.set_title(name)
        self.set_name(name)
    
        self.set_border_width(0)
        self.set_usize(600, 400)

        self.box1 = GtkVBox()
        self.add(self.box1)
        self.box1.show()

        # start up a menu
        self.init_menus()
        
        self.box2 = GtkVBox(spacing=10)
        self.box2.set_border_width(5)
        self.box1.pack_start(self.box2)
        self.box2.show()

        self.notebook = GtkNotebook()
        self.notebook.set_tab_pos(POS_TOP)
        self.box2.pack_start(self.notebook)
        self.notebook.show()

        # start the main log window
        self.init_main_log()
        
        self.show()

##        self.log_print("Hello.")
##        for i in range(30):
##            self.log_print("Have a supercalifragiliciousexpialidocious day! And on and so on and so forth, etc, etc, ad infinitum, and so on.")

    def init_menus(self):
        self.hdlbox = GtkHandleBox()
        self.box1.pack_start(self.hdlbox, expand=FALSE)
        self.hdlbox.show()
        self.mf = GtkExtra.MenuFactory()
        self.mf.add_entries([
            ('File/Open...', '<control>O', self.file_open),
            ('File/Save As...', '<control>S', self.file_save_as),
            ('File/<separator>', None, None),
            ('File/Exit', '<control>Q', self.file_exit),
            ('Help/About...', None, self.help_about),
        ])
        self.add_accel_group(self.mf.accelerator)        
        self.hdlbox.add(self.mf)
        self.mf.show()
        
            
    def init_main_log(self):

        self.main_window = GtkHBox()
        self.main_window.set_border_width(5)
        self.main_window.show()

        self.main_text=GtkText()
        
        self.main_text.set_editable(FALSE)
        self.main_text.set_word_wrap(FALSE)
        self.main_text.set_line_wrap(FALSE)
        self.main_window.pack_start(self.main_text, padding=1)
        self.main_text.show()

##        self.hscroll = GtkHScrollbar(self.main_text.get_hadjustment())
##        self.hscroll.set_update_policy(POLICY_AUTOMATIC)
##        self.main_window.pack_start(self.hscroll, expand=FALSE)

##        self.hscroll.show()
                
        self.vscroll = GtkVScrollbar(self.main_text.get_vadjustment())
        self.vscroll.set_update_policy(POLICY_AUTOMATIC)
        self.main_window.pack_end(self.vscroll, expand=FALSE)
        self.vscroll.show()
    
        # create label for main notebook tab
        self.main_tab_label = GtkLabel("Main Log")
        self.main_tab_label.set_padding(1,1)        
        
        # add main text box to notebook        
        self.notebook.append_page(self.main_window, self.main_tab_label)
        
    def main_log_hook(self, log_string):
        # print "in main_log_hook"
        self.log_print(log_string)
        return
            
    def file_open(self, args=None, fname=None):
        # print "This is file open"
        if not fname:
            fname = GtkExtra.file_open_box(modal=FALSE)
        if fname:
            SubmitList = []
            file_handle = open(fname, "r")
            for line in file_handle.readlines():
                if line != "\n":
                    # remove blank lines and strip excess whitespace
                    SubmitList.append(string.strip(line))

        pagecast = Pagecast(SubmitList, None, self.main_log_hook, None)
        pagecast.start()
        
        
    def file_save_as(self, args):
        print "This is file save as"

    def file_exit(self, args):
        self.quit(None)

    def help_about(self, mi=None):
        About_Text = """Pagecast 1.1 -- GTK+ Interface
By Preston Landers <preston@askpreston.com>
Distributed under the GNU General Public License

Brought to you by the letters C, H, and T, and the number 420."""

        GtkExtra.message_box("Drop", About_Text, ("Far Out!",))

    def quit(self, args, mi=None):
        self.destroy()
        mainquit()

    def log_print(self, String):
        # print "Printing string: " + String
        self.main_text.insert_defaults(String + "\n")
        
if __name__ == '__main__':
    import sys
    
    # check if to load document on startup
    if len(sys.argv) > 1:
        FileRequest = sys.argv[1]
    else:
        FileRequest = None    
    
    app = Application('Pagecast Control Center')
    app.show()

    if FileRequest:
        app.file_open(None, FileRequest)
    
    mainloop()
