#!/usr/bin/env python

### Copyleft (C) 2000 Preston B. Landers <pbl@pbl.cx>

### See file COPYING for License and Warranty information
### See file README for instructions
### See file "pagecast" for other comments

### developed with Python 1.5.2 on Debian 2.2 (Linux 2.2.x)

###
### VERSION 2.0 beta 1 January 2000
###


"""
submitengine -- a library used by Pagecast to do most of its work

VERSION 2.0 beta 1 January 2000

It is *not* a good idea to use 'from submitengine import *' with this module.

Use this instead:

>>> import submitengine
>>> se = submitengine.submitengine(args)
>>> se.Go()

Copyleft (C) 2000 Preston Landers <pbl@pbl.cx>
"""

from threading import *      # threads are required
import time, string, os      # standard stuff
import ConfigParser          # to process .conf configuration files
import metatag               # custom library to help evaluate meta tags
import lumberjack            # custom logging facility
import sanitize              # custom fixes broken links before submitting
import smtp                  # custom wrapper around smtplib 

# Load translation tables
try:
    execfile("src/translate.py")
except: 
    print "Error loading translation tables in submitengine.py! Aborting. :-("
    raise SystemExit(1)  # crater

class Globals:
    """Globals: a class for holding global variables.
Globals is a class to contain (primarily configuration) information
to be used throughout the program.  Also contains the logging object."""
    
    def __init__(self, Verbosity = 0):
        self.version = _("version")
        self.homedir = "/home/pagecast"
        self.servers = "servers/"
        self.copyright = _("copyright")
        self.configfiles = ["etc/pagecast.conf"]
        self.serverdefs = ["etc/servers.conf"]
        self.default_groups = ["us", "english", "general"]
        self.tagsdefs = ["etc/tags.conf"]
        self.groupsfile = "etc/groups.conf"
        self.log = None
        self.proxy = None
        self.proxyport = None
        self.verbosity = Verbosity

    def pprint(self, string, verbosity_level = 0):
        if verbosity_level <= self.verbosity:
            """useful for printing & logging debugging statements"""
            if self.log:
                self.log.add(self.log.Log(string))
                # *always* print to stdout (unless log already does it)
                if not self.log.stdoutput:
                    print _(string)
            else:
                print _(string)


###
### begin support functions
###

        
def Mail_Reply(G, config, ReplyTo):
    """Mail_Reply: a function to handle mailing a response back to the user.
The parameters are:
  config: a ConfigParser with all the 'general' config info
  ReplyTo: the mail address (in 'joe@somewhere.com' format) to mail to
Returns None if everything went okay."""
    try:
        Server = config.get("Mail","smtp")
        From = config.get("Mail", "from")
        Subject = config.get("Mail","responsesubject")        
    except:
        print _("bad_mail_configure")
        return 1

    try:
        CC = config.get("Mail","cc")
    except:
        CC = None

    try:
        BCC = config.get("Mail","bcc")
    except:
        BCC = None

    Body = string.join(self.G.log.log_list)


    mail_handler = smtp.mail_handler(Server, From)

    # return None if everything OK
    return mail_handler.send_email(To, Subject, Body, CC, BCC)


###    
### main object begins here
###

class submitengine:

    def __init__(self, SubmitList, Verbosity = 0, ReplyTo = None, Hooks = None, SpecifiedGroups = None):
        """submitengine.Go(SubmitList, ReplyAddress) -- Makes the whole show run.
SubmitList is a list of URLs.
If ReplyAddress is None, then no response will be mailed back to a user.
Hooks is an optional list of hook functions to be added to the log."""

        self.G = Globals(Verbosity)
        self.ReplyTo = ReplyTo
        self.SubmitList = SubmitList

        # parse configuration files (general config and servers)
        self.config = ConfigParser.ConfigParser()
        self.config.read(self.G.configfiles)
        
        try:
            # Enable logging if user requested
            logtoggle = self.config.get("Log","log")
            if logtoggle != "on":
                logfile = None
            else:
                logfile = self.config.get("Log","logfile")

            # get Global config info
            self.G.email = self.config.get("Mail","email")
            self.G.smtp_server = self.config.get("Mail", "smtp_server")
            self.G.retries = self.config.getint("Network","retries")
            self.G.output = self.config.get("General","output")
            self.G.actionwait = self.config.getfloat("Network","wait")
            self.G.fixkeywords = self.config.get("General","fixkeywords")
            self.G.confidence_threshold = self.config.getint("General","confidencethreshold")
            self.G.confidence_print = self.config.get("General","printconfidence")
            self.G.localdocuments = self.config.get("General","localdocuments")
        except:
            self.G.pprint(_("config_parse_error"))
            raise SystemExit(1)

        # start the logger w/ no 'debugging' mode
        self.G.log = lumberjack.Lumberjack(logfile, self.G.output, "keep_internal_log", Hooks, None) 
        
        # try configuring proxy support
        try:
            self.G.proxy = self.config.get("Network","proxy")
            self.G.proxyport = self.config.getint("Network","proxyport")
        except ConfigParser.NoOptionError:
            self.G.proxy = None
            self.G.proxyport = None

        # try configuring maximum threads
        # (max # of simulatneous connections)
        try:
            self.MaxThreads = self.config.getint("Network","MaxThreads")
        except:
            self.MaxThreads = None
            

        ### CONFIGURE GROUPS

        # Active is a list of the groups that will actually be 'activated' and run
        # Assign maps Server names to a list of group names
        # (it is a dict where the key is a server.name and the value is a list of group.name)
        Active = []
        Assign = {}
        
        execfile(self.G.groupsfile, globals(), locals())
        
        if SpecifiedGroups:
            Active = SpecifiedGroups
            self.G.pprint(_("d_specified_groups") % (SpecifiedGroups), 1)

        ##
        ## Handle server drivers
        ## 
        
        # get a list of files in the servers directory        
        Servers = []
        try:
            for file in os.listdir(self.G.servers):
                if file[-4:] == ".def":                    
                    Servers.append(file)
                    self.G.pprint("Added server definition file %s" % (file), 2)
        except OSError:
            self.G.pprint(_("server_parse_error"))
            raise SystemExit

        ### ServerRegister is a dictionary where the keys are the unique
        ### text names of each server, and the values are references to
        ### the Server Thread Objects.
        ServerRegister = {}
        
        G = self.G
        class Configuration: pass
        count = 1
        for file in Servers:
            count = count + 1
            # TODO insert security here
            execfile("%s%s" % (self.G.servers, file), globals(), locals())            

        ### Assign groups to Server objects
        for server in ServerRegister.keys():
            try:                
                ServerRegister[server].groups = Assign[server]
            except:
                ServerRegister[server].groups = self.G.default_groups
                self.G.pprint(_("default_groups_msg") % (server, self.G.default_groups), 1)
            
        self.ServerRegister = {}

        # trim out inactive groups/servers
        for servername in ServerRegister.keys():
            server = ServerRegister[servername]
            for Group in server.groups:
                if Group in Active:
                    self.ServerRegister[servername] = server
                    break
                
        self.G.pprint(_("d_active_groups") % (str(Active)), 2)
        for key in self.ServerRegister.keys():
            self.G.pprint(_("d_server_activated") % (key, self.ServerRegister[key].groups), 2)                            

    def Go(self):
        ### prepare BEGIN message
        the_time = time.localtime(time.time())        
        str_time = str(the_time[0]) + "-" + str(string.zfill(the_time[1],2)) + "-" + str(string.zfill(the_time[2],2)) + " " + str(string.zfill(the_time[3],2)) + ":" + str(string.zfill(the_time[4],2)) + ":" + str(string.zfill(the_time[5],2))

        ### print BEGIN message
        self.G.pprint(_("begin_run_msg") % (str_time, self.G.version ))
        self.G.pprint(_("copyright"))

        # Pretty up the URLs (if desired)
        if self.config.get("General", "fixlinks") == "on":
            Strict = None
            if self.config.get("General", "strictlinks") == "on":
                Strict = 1
                
            new_list = []
            for link in self.SubmitList:
                new_link = sanitize.Pretty_Link(link, Strict)
                if new_link:
                    new_list.append(new_link)
                else:
                    self.G.pprint(_("broken_link") % link)
                    
            self.SubmitList = new_list

        ### see if we want to use meta tag magic
        if self.config.get("General", "tags") == "on":
            metatag.MetaTagMagic(G,self.config,self.SubmitList)
            
        ### prepare to start server threads
        started = activeCount()
        initial_thread_count = started
        thread_limit = started + self.MaxThreads
        self.G.pprint("DEBUG Maximum threads: %d, %d threads already started" % (thread_limit, started), 3)

        done = None

        # start threads        
        while started <  thread_limit and not done:
            for server in self.ServerRegister.keys():
                Server = self.ServerRegister[server]
                if Server.status == "init":
                    Server.status = "started"
                    started = started + 1
                    # Do it!
                    Server.Go(self.SubmitList[:])
                    self.G.pprint("DEBUG started server %s thread %d limit %d" % (server, started, thread_limit), 3)
                    break
            else:
                # we must have started all threads!
                done = 1

        # poll threads until finished
        busy = Event()
        try:
            while activeCount() > initial_thread_count:
                busy.wait(1.1)  # check on active threads every second or so

                # if we have yet-unstarted threads left, start one
                if activeCount() < thread_limit:
                    for server in self.ServerRegister.keys():
                        Server = self.ServerRegister[server]
                        if Server.status == "init":
                            started = started + 1
                            # Do it!
                            Server.Go(self.SubmitList[:])
                            self.G.pprint("started NEW server %s #%d" % (server, started), 3)
                            break

        except:
            self.G.pprint(_("user_quit"))
            
            for eachthread in self.ServerRegister.keys():
                self.ServerRegister[eachthread].exitflag = 1
            raise SystemExit
        
        
        # calculate stats
        totalhits = 0
        URLcount = len(self.SubmitList)
        successful = 0
        errors = 0
        num_servers = 0

        for server in self.ServerRegister.keys():
            ThreadObject = self.ServerRegister[server]
            try:
                if ThreadObject.status is "done":
                    num_servers = num_servers + 1

                    totalhits = totalhits + ThreadObject.hitcount
                    successful = successful + ThreadObject.successcount
                    errors = errors + ThreadObject.errorcount
            except:
                totalhits = -1
                successful = -1
                errors = -1

        # print summary messages
        self.G.pprint(_("summary") % (URLcount, num_servers, successful, errors))
                
        # clean up; print END messages
        self.G.pprint(_("done_message"))

        # send all the results if neccesary
        if self.ReplyTo:        
            Mail_Reply(self.config, self.ReplyTo)

if __name__ == "__main__":
    print "This is a module; do not run it directly.  Please read the documentation."
    raise SystemExit(1)
